<?php

namespace SenSai;

use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Control;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Manialinks\IconManager;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Commands\CommandListener;

/**
 * Elite Extra Plugin
 *
 * @author Sen-Sai
 */

class EliteX implements CallbackListener, TimerListener, ManialinkPageAnswerListener, CommandListener, Plugin {
    /**
     * Constants
     */
    const PLUGIN_ID			= 68;
    const PLUGIN_VERSION	= 1;
    const PLUGIN_NAME		= 'Elite Extra';
    const PLUGIN_AUTHOR		= 'G]R [SG-1]Sen-sai';

    // Loadscreen properties

    const MLID_WIDGET				= 'EliteX.ID';
    const SETTING_ELITEX_LOGO		= 'Logo URL';
	const ELITEX_IMG_RIP			= 'rip.png';
	const ELITEX_IMG_MIN			= 'min.png';
	const ELITEX_IMG_MAX			= 'max.png';
	const ACTION_MINIMIZE_WIDGET	= 'EliteX.Minimize';
	const ACTION_MAXIMIZE_WIDGET  	= 'EliteX.Maximize';

	private $img = array();
	private $teams = array();
	private $playing = false;
	private $widget = array();

    /**
     * Private Properties
     */
    /**
     * @var maniaControl $maniaControl
     */
    private $maniaControl = null;


    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     * @return mixed
     */
    public static function prepare(ManiaControl $maniaControl) {
        //do nothing
    }

    /**
     * Load the plugin
     *
     * @param ManiaControl $maniaControl
     * @return bool
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;
		if ($this->maniaControl->server->titleId != 'SMStormElite@nadeolabs') {
			$error = 'This plugin only supports the Elite Mode (check Settings)!';
			throw new \Exception($error);
		}

        //Register callbacks

        $this->maniaControl->settingManager->initSetting($this, self::SETTING_ELITEX_LOGO, 'http://files.designburo.nl/shootmania/images/glrimratten.png');

		$file = $this->maniaControl->settingManager->getSetting($this, self::SETTING_ELITEX_LOGO);
		$array = get_headers($file);
		$string = $array[0];
		if(!strpos($string,"200")) {
			
			$this->maniaControl->chat->sendInformation('The URL does not exist. Please check. Elite Extra Plugin disabled.', null, false);
			return;
		}
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACK, $this, 'handleCallbacks');
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_MODESCRIPTCALLBACKARRAY, $this, 'handleCallbacks');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERINFOCHANGED, $this, 'handlePlayerInfoChangedCallback');
		$this->maniaControl->callbackManager->registerCallbackListener(CallbackManager::CB_MP_PLAYERMANIALINKPAGEANSWER, $this, 'handleManialinkPageAnswer');
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ACTION_MINIMIZE_WIDGET, $this, 'minimizeWidget');
		$this->maniaControl->manialinkManager->registerManialinkPageAnswerListener(self::ACTION_MAXIMIZE_WIDGET, $this, 'maximizeWidget');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnectCallback');
		$this->maniaControl->callbackManager->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'updatePlayerLeft');
		$this->maniaControl->commandManager->registerCommandListener('hidewidget', $this, 'hide', true);
		$this->maniaControl->commandManager->registerCommandListener('showwidget', $this, 'show', true);
		
		$ext = substr(strrchr($file, "."), 1);
		if ($ext=="jpg" || $ext=="png")
		{
			
	
			$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::ELITEX_IMG_RIP,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::ELITEX_IMG_MIN,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->manialinkManager->getIconManager()->addIcon(self::ELITEX_IMG_MAX,'http://files.designburo.nl/shootmania/images/');
			$this->img['logo']=$file;
			
		}
		else
		{
			$this->maniaControl->chat->sendInformation('The URL must be either .png or .jpg', null, false);
			$this->maniaControl->callbackManager->unregisterCallbackListener($this);
			return;	
		}
		
		$this->teams['defender'][0]["status"]="empty";
		$this->teams['defender'][1]["status"]="empty";
		$this->teams['defender'][2]["status"]="empty";
		$this->teams['attacker']="";
		foreach($this->maniaControl->playerManager->getPlayers() as $player) {
		$this->widget[$player->login]="max";	
		}
		
		//$this->maniaControl->timerManager->registerTimerListening($this, 'handle1Second', 1000);
    }

	public function minimizeWidget(array $chatCallback, Player $player)
	{
		$this->widget[$player->login]="min";
		$this->displayTurnWidget($player->login,$this->widget[$player->login]);		
	}

	public function maximizeWidget(array $chatCallback, Player $player)
	{
		$this->widget[$player->login]="max";
		$this->displayTurnWidget($player->login,$this->widget[$player->login]);
		
	}
	
	public function handleManialinkPageAnswer(array $callback) {
		$actionId    = $callback[1][2];
		$actionArray = explode('.', $actionId);
		if (count($actionArray) <= 2) {
			return;
		}

		$action = $actionArray[0] . '.' . $actionArray[1];
		if ($action !== "doit") return;
	}
	
	
/*
	public function handleManialinkPageAnswer(array $callback) {
		$actionId    = $callback[1][2];
		$actionArray = explode('.', $actionId);
		if (count($actionArray) <= 2) {
			return;
		}
		/*	const ACTION_MINIMIZE_WIDGET	= 'EliteX.Minimize';
			const ACTION_MAXIMIZE_WIDGET  	= 'EliteX.Maximize';
		*/
/*
		$action = $actionArray[0] . '.' . $actionArray[1];
		//if ($action !== self::ACTION_SOUNDBOXPLAY) return;
		$login  = $callback[1][1];
		//$player = $this->maniaControl->playerManager->getPlayer($login);
		$this->maniaControl->chat->sendInformation('action :'.$action, null, false);
		if ($action == self::ACTION_MAXIMIZE_WIDGET)
		{
			$this->widget[$login]="max";
		}
		if ($action == self::ACTION_MINIMIZE_WIDGET)
		{
			$this->widget[$login]="min";
		}
		$this->displayTurnWidget($login,$this->widget[$login]);
		
	}
*/
	public function handlePlayerConnectCallback(Player $player)
	{
		if($this->playing==false) return;
		$this->widget[$player->login]="max";
		$this->displayTurnWidget($player->login,$this->widget[$player->login]);
		
	}


	public function handlePlayerInfoChangedCallback(Player $player)
	{
		if($this->playing==false) return;
		$login = $player->login;	
		if ($this->teams['defender'][0]["status"]=="playing")
		{
			if($login==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="afk";	
				$this->widgetChoice();
			}
		}
		
		if ($this->teams['defender'][1]["status"]=="playing")
		{
			if($login==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="afk";	
				$this->widgetChoice();
			}
		}
		
		if ($this->teams['defender'][2]["status"]=="playing")
		{
			if($login==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="afk";	
				$this->widgetChoice();
			}
		}
		
		
	}

	public function updatePlayerLeft(Player $player)
	{
		if($this->playing==false) return;
		$login = $player->login;
		$this->widget[$login]="left";
		if ($this->teams['defender'][0]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="left";	
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][1]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="left";	
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][2]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="left";	
				$this->widgetChoice();
			}
		}
		
	}

	public function updateKill($info)
	{
		if($this->playing==false) return;
		$victim = $info["Event"]["Victim"]["Login"];
		if ($this->teams['defender'][0]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="dead";	
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][1]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="dead";
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][2]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="dead";
				$this->widgetChoice();
			}
		}
	}
	
	public function startStatus($info)
	{

		if ($info) {
			if(isset($info["DefendingPlayers"][0]["Login"])){
				$this->teams['defender'][0]["login"]=$info["DefendingPlayers"][0]["Login"];
				$this->teams['defender'][0]["status"]="playing";
				$login = $this->teams['defender'][0]["login"];
				
			}
			else
			{
				$this->teams['defender'][0]["status"]="empty";
			}
			if(isset($info["DefendingPlayers"][1]["Login"])){
				$this->teams['defender'][1]["login"]=$info["DefendingPlayers"][1]["Login"];
				$this->teams['defender'][1]["status"]="playing";
				$login = $this->teams['defender'][1]["login"];
				
			}
			else
			{
				$this->teams['defender'][1]["status"]="empty";
			}
			if(isset($info["DefendingPlayers"][2]["Login"])){
				$this->teams['defender'][2]["login"]=$info["DefendingPlayers"][2]["Login"];
				$this->teams['defender'][2]["status"]="playing";
				$login = $this->teams['defender'][2]["login"];
				
			}
			else
			{
				$this->teams['defender'][2]["status"]="empty";
			}
			
			$this->teams['attacker']=$info["AttackingPlayer"]["Name"];
			
			
			
		}
		else
		{
			$this->maniaControl->chat->sendInformation('No Players', null, false);
			$this->hide();
			return;
			
		}
		$this->widgetChoice();
	}

	public function widgetChoice()
	{

		foreach($this->widget as $login => $action)
		{
			if($this->widget[$login]!=="left")
			{
				$this->displayTurnWidget($login,$action);
			}
		}
		
		
	}

	public function displayTurnWidget($to=false,$action){
		
		/*	const ACTION_MINIMIZE_WIDGET	= 'EliteX.Minimize';
			const ACTION_MAXIMIZE_WIDGET  	= 'EliteX.Maximize';
		*/
		//if ($action !== self::ACTION_SOUNDBOXPLAY) return;
		//setAction(self::ACTION_OPEN_SOUNDBOX);
		
        $manialink = new ManiaLink(self::MLID_WIDGET);		
		
		$x=141;
		$y=10;
		if($action=="max")
		{
			//background
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(35,50);
			$quad->setPosition($x,$y);
			$quad->setBgColor("000");
			$quad->setZ(2);
			$quad->setOpacity(0.5);
			//$quad->setAction(self::ACTION_MINIMIZE_WIDGET);
			//Title
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(35,10);
			$quad->setPosition($x,$y+30);
			$quad->setBgColor("00F");
			$quad->setZ(2);
			$quad->setOpacity(0.5);
			//logo
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(10,10);
			$quad->setPosition($x-12.5,$y+30);
			$quad->setImage($this->img['logo']);
			//$quad->setAction(self::ACTION_MINIMIZE_WIDGET);
			$quad->setZ(3);
			
			//Minimize
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(5,5);
			$quad->setPosition($x+15,$y+32.5);
			$quad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::ELITEX_IMG_MIN));
			$quad->setAction(self::ACTION_MINIMIZE_WIDGET);
			$quad->setZ(5);
			
			//title
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x-17.5+12,$y+28);
			$label->setZ(3);
			$label->setText('$o$fffRound Info');
			$label->setTextSize(1);
			$label->setHAlign(Control::LEFT);
			
		
		}
		if($action=="min")
		{
			//background
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(35,10);
			$quad->setPosition($x,$y+30);
			$quad->setBgColor("000");
			$quad->setZ(2);
			$quad->setOpacity(0.5);	
			//$quad->setAction(self::ACTION_MAXIMIZE_WIDGET);
			//Title
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(35,10);
			$quad->setPosition($x,$y+30);
			$quad->setBgColor("00F");
			$quad->setZ(2);
			$quad->setOpacity(0.5);
			//logo
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(10,10);
			$quad->setPosition($x-12.5,$y+30);
			$quad->setImage($this->img['logo']);
			//$quad->setAction(self::ACTION_MAXIMIZE_WIDGET);
			$quad->setZ(3);	
			
			//Minimize
			$quad = new Quad();
			$manialink->add($quad);
			$quad->setSize(5,5);
			$quad->setPosition($x+15,$y+32.5);
			$quad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::ELITEX_IMG_MAX));
			$quad->setAction(self::ACTION_MAXIMIZE_WIDGET);
			$quad->setZ(5);	
			
			//title
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x-17.5+12,$y+28);
			$label->setZ(3);
			$label->setText('$o$fffRound Info');
			$label->setTextSize(1);
			$label->setHAlign(Control::LEFT);	
			
		}
		
		if($action=="max")
		{

			
		
			
			//Attacker Title
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x,$y+20);
			$label->setZ(2);
			$label->setText('$o$ff0$sATTACKER');
			$label->setTextSize(1);
			$label->setHAlign(Control::CENTER);
			
			//Attacker 
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x,$y+15);
			$label->setZ(2);
			$label->setText('$o$f00'.$this->teams['attacker']);
			$label->setTextSize(1);
			$label->setHAlign(Control::CENTER);
			
		
			//Defenders Title
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x,$y+5);
			$label->setZ(2);
			$label->setText('$o$ff0$sDEFENDERS');
			$label->setTextSize(1);
			$label->setHAlign(Control::CENTER);
			
			
			if($this->teams['defender'][0]["status"]=="dead") {
			//RIP player 1
				$quad = new Quad();
				$manialink->add($quad);
				$quad->setSize(8,8);
				$quad->setPosition($x-13.5,$y-2);
				$quad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
				$quad->setZ(2);
				//$quad->setOpacity(0.7);
			}
			if($this->teams['defender'][1]["status"]=="dead") {
				//RIP player 2
				$quad = new Quad();
				$manialink->add($quad);
				$quad->setSize(8,8);
				$quad->setPosition($x-13.5,$y-10);
				$quad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
				$quad->setZ(2);
				//$quad->setOpacity(0.7);
			}
			if($this->teams['defender'][2]["status"]=="dead") {
				//RIP player 3
				$quad = new Quad();
				$manialink->add($quad);
				$quad->setSize(8,8);
				$quad->setPosition($x-13.5,$y-18);
				$quad->setImage($this->maniaControl->manialinkManager->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
				$quad->setZ(2);
				//$quad->setOpacity(0.7);
			}
			
	
			//Defender 1 
			(string)$name="";
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x-12,$y-2);
			$label->setZ(2);
			switch ($this->teams['defender'][0]["status"])
			{
				case "left":
				(string)$name = $this->teams['defender'][0]["login"];
				
				$label->setText('$999(left)$z$999$n '.$name);
				break;
				
				case "afk":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][0]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999(afk)$z$999$n '.$name);
				break;
				
				case "dead":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][0]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999$n '.$name);
				$label->setPosition($x-7,$y-2);
				break;
				
				case "empty":
				$label->setText(' ');
				break;
				
				case "playing":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][0]["login"]);
				$name = $player->nickname;
				$label->setText('$f00'.$name);
				break;
			}
			$label->setTextSize(1);
			$label->setHAlign(Control::LEFT);
			
			//Defender 2
			$name="";
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x-12,$y-10);
			$label->setZ(2);
			switch ($this->teams['defender'][1]["status"])
			{
				case "left":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][1]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999(left)$z$999$n '.$name);
				break;
				
				case "afk":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][1]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999(afk)$z$999$n '.$name);
				break;
				
				case "dead":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][1]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999$n '.$name);
				$label->setPosition($x-7,$y-10);
				break;
				
				case "empty":
				$label->setText(' ');
				break;
				
				case "playing":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][1]["login"]);
				$name = $player->nickname;
				$label->setText('$f00'.$name);
				break;
			}
			$label->setTextSize(1);
			$label->setHAlign(Control::LEFT);
			
			//Defender 3 
			$name="";
			$label = new Label_Text();
			$manialink->add($label);
			$label->setPosition($x-12,$y-18);
			$label->setZ(2);
			switch ($this->teams['defender'][2]["status"])
			{
				case "left":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][2]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999(left)$z$n$999 '.$name);
				break;
				
				case "afk":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][2]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999(afk)$z$n$999 '.$name);
				break;
				
				case "dead":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][2]["login"]);
				$name = $player->nickname;
				
				$label->setText('$999$n '.$name);
				$label->setPosition($x-7,$y-18);
				break;
				
				case "empty":
				$label->setText(' ');
				break;
				
				case "playing":
				$player = $this->maniaControl->playerManager->getPlayer($this->teams['defender'][2]["login"]);
				$name = $player->nickname;
				$label->setText('$f00'.$name);
				break;
			}
			$label->setTextSize(1);
			$label->setHAlign(Control::LEFT);
		}
		
		if(!$to)
		{
			$this->maniaControl->manialinkManager->sendManialink($manialink);
		}
		else
		{
			$player = $this->maniaControl->playerManager->getPlayer($to);
			$this->maniaControl->manialinkManager->sendManialink($manialink,$player);
			
		}
		
	}
   
	
    public function hide() {
        	$emptyManialink = new ManiaLink(self::MLID_WIDGET);
			$emptyManialink->removeChildren();
        	$this->maniaControl->manialinkManager->sendManialink($emptyManialink);
			
    }
	
    public function show() {
			$this->widgetChoice();
    }



	public function handleCallbacks(array $callback) {
		$callbackName = $callback[1][0];
	
		switch($callbackName) {
			
			// ROYAL
		case 'BeginTurn':
			$data = $callback[1][1];
			$info=json_decode($data, true);
			$this->startStatus($info);
			$this->playing=true;
		break;
		
		case 'EndTurn':
			//$this->hide();
			$this->playing=false;
		break;
		
		case 'EndMap':
			$this->hide();
			$this->playing=false;
		break;
		
		case 'BeginWarmup':
			$this->hide();
			$this->playing=false;
		break;

		case 'OnArmorEmpty':
			$data = $callback[1][1];
			$info=json_decode($data, true);
			$this->updateKill($info);
		break;
		}
	}


    public function unload() {
        $this->hide();
        $this->maniaControl->callbackManager->unregisterCallbackListener($this);
        unset($this->maniaControl);
    }

     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::PLUGIN_ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::PLUGIN_NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::PLUGIN_VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::PLUGIN_AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return 'Displays round start info.';
    }
}